@@ -3,7 +3,6 @@ Implements a client class to query the
 `Deezer API <http://developers.deezer.com/api>`_
 """
 import warnings
-from urllib.parse import urlencode
 
 import requests
 
@@ -12,13 +11,15 @@ from deezer.resources import (
     Artist,
     Chart,
     Comment,
+    Episode,
     Genre,
     Playlist,
+    Podcast,
     Radio,
+    Resource,
     Track,
     User,
 )
-from deezer.utils import SortedDict
 
 DEPRECATED_ARG_MESSAGE = (
     "The `{arg_name}` keyword argument is deprecated "
@@ -52,25 +53,27 @@ class Client:
 
     .. deprecated:: 1.4.0
 
-    The following parameters will be removed in the next major version:
+        The following parameters will be removed in the next major version:
 
-        * **host** - override the default hostname.
-        * **use_ssl** - connect using HTTP if set to `False`.
+            * **host** - override the default hostname.
+            * **use_ssl** - connect using HTTP if set to `False`.
     """
 
     objects_types = {
         "album": Album,
         "artist": Artist,
+        "chart": Chart,
         "comment": Comment,
         "editorial": None,
+        "episode": Episode,
         # 'folder': None, # need identification
         "genre": Genre,
         "playlist": Playlist,
+        "podcast": Podcast,
         "radio": Radio,
         "search": None,
         "track": Track,
         "user": User,
-        "chart": Chart,
     }
 
     def __init__(
@@ -123,7 +126,11 @@ class Client:
             result[parent.type] = parent
 
         if "type" in result:
-            object_class = self.objects_types[result["type"]]
+            if result["type"] in self.objects_types:
+                object_class = self.objects_types[result["type"]]
+            else:
+                # in case any new types are introduced by the API
+                object_class = Resource
         else:
             object_class = self.objects_types[parent]
         return object_class(self, result)
@@ -135,13 +142,11 @@ class Client:
         """
         return self.use_ssl and "https" or "http"
 
-    def url(self, request=""):
+    def url(self, request_path=""):
         """Build the url with the appended request if provided."""
-        if request.startswith("/"):
-            request = request[1:]
-        return "{}://{}/{}".format(self.scheme, self.host, request)
+        return f"{self.scheme}://{self.host}/{request_path}"
 
-    def object_url(self, object_t, object_id=None, relation=None, **kwargs):
+    def object_url(self, object_t, object_id=None, relation=None):
         """
         Helper method to build the url to query to access the object
         passed as parameter
@@ -149,43 +154,48 @@ class Client:
         :raises TypeError: if the object type is invalid
         """
         if object_t not in self.objects_types:
-            raise TypeError("{} is not a valid type".format(object_t))
+            raise TypeError(f"{object_t} is not a valid type")
         request_items = (
             str(item) for item in [object_t, object_id, relation] if item is not None
         )
-        request = "/".join(request_items)
-        base_url = self.url(request)
-        if self.access_token is not None:
-            kwargs["access_token"] = str(self.access_token)
-        if kwargs:
-            for key, value in kwargs.items():
-                if not isinstance(value, str):
-                    kwargs[key] = str(value)
-            # kwargs are sorted (for consistent tests between Python < 3.7 and >= 3.7)
-            sorted_kwargs = SortedDict.from_dict(kwargs)
-            result = "{}?{}".format(base_url, urlencode(sorted_kwargs))
-        else:
-            result = base_url
-        return result
+        request_path = "/".join(request_items)
+        return self.url(request_path)
 
     def get_object(
-        self, object_t, object_id=None, relation=None, parent=None, **kwargs
+        self, object_t, object_id=None, relation=None, parent=None, **params
     ):
         """
-        Actually query the Deezer API to retrieve the object
+        Actually query the Deezer API to retrieve the object.
 
-        :returns: json dictionary
+        :returns: an :class:`~deezer.resources.Resource` or subclass.
         """
-        url = self.object_url(object_t, object_id, relation, **kwargs)
-        response = self.session.get(url)
-        json = response.json()
-        if "error" in json:
+        url = self.object_url(object_t, object_id, relation)
+        params = params or {}
+        if self.access_token is not None:
+            params["access_token"] = str(self.access_token)
+        response = self.session.get(url, params=params)
+        json_data = response.json()
+        if "error" in json_data:
             raise ValueError(
-                "API request return error for object: {} id: {}".format(
-                    object_t, object_id
-                )
+                f"API request return error for object: {object_t} id: {object_id}"
             )
-        return self._process_json(json, parent)
+        return self._process_json(json_data, parent)
+
+    def get_album(self, object_id, relation=None, **kwargs):
+        """
+        Get the album with the provided ID.
+
+        :returns: an :class:`~deezer.resources.Album` object
+        """
+        return self.get_object("album", object_id, relation=relation, **kwargs)
+
+    def get_artist(self, object_id, relation=None, **kwargs):
+        """
+        Get the artist with the provided ID.
+
+        :returns: an :class:`~deezer.resources.Artist` object
+        """
+        return self.get_object("artist", object_id, relation=relation, **kwargs)
 
     def get_chart(self, relation=None, index=0, limit=10, **kwargs):
         """
@@ -197,22 +207,6 @@ class Client:
             "chart", object_id="0", relation=relation, parent="chart", **kwargs
         )
 
-    def get_album(self, object_id, relation=None, **kwargs):
-        """
-        Get the album with the provided id
-
-        :returns: an :class:`~deezer.resources.Album` object
-        """
-        return self.get_object("album", object_id, relation=relation, **kwargs)
-
-    def get_artist(self, object_id, relation=None, **kwargs):
-        """
-        Get the artist with the provided id
-
-        :returns: an :class:`~deezer.resources.Artist` object
-        """
-        return self.get_object("artist", object_id, relation=relation, **kwargs)
-
     def get_comment(self, object_id):
         """
         Get the comment with the provided id
@@ -221,6 +215,14 @@ class Client:
         """
         return self.get_object("comment", object_id)
 
+    def get_episode(self, object_id):
+        """
+        Get the episode with the provided id
+
+        :returns: a :class:`~deezer.resources.Episode` object
+        """
+        return self.get_object("episode", object_id)
+
     def get_genre(self, object_id):
         """
         Get the genre with the provided id
@@ -243,6 +245,14 @@ class Client:
         """
         return self.get_object("playlist", object_id)
 
+    def get_podcast(self, object_id):
+        """
+        Get the podcast with the provided id
+
+        :returns: a :class:`~deezer.resources.Podcast` object
+        """
+        return self.get_object("podcast", object_id)
+
     def get_radio(self, object_id=None):
         """
         Get the radio with the provided id.
@@ -303,13 +313,15 @@ class Client:
         :returns: a list of :class:`~deezer.resources.Resource` objects.
 
         >>> client.advanced_search({"artist": "Daft Punk", "album": "Homework"})
-        >>> client.advanced_search({"artist": "Daft Punk", "album": "Homework"},
-        ...                        relation="track")
+        >>> client.advanced_search(
+        ...     {"artist": "Daft Punk", "album": "Homework"},
+        ...     relation="track",
+        ... )
         """
         if not isinstance(terms, dict):
             raise TypeError("terms must be a dict")
         # terms are sorted (for consistent tests between Python < 3.7 and >= 3.7)
-        query = " ".join(sorted(['{}:"{}"'.format(k, v) for (k, v) in terms.items()]))
+        query = " ".join(sorted(f'{k}:"{v}"' for (k, v) in terms.items()))
         return self.get_object(
             "search", relation=relation, q=query, index=index, limit=limit, **kwargs
         )
